@@ -39,36 +39,39 @@
 #include "Peer.h"
 #include "DownloadContext.h"
 #include "Logger.h"
+#include "wallclock.h"
+#include "util.h"
 
 namespace aria2 {
 
 BtStopDownloadCommand::BtStopDownloadCommand
-(int32_t cuid,
+(cuid_t cuid,
  RequestGroup* requestGroup,
  DownloadEngine* e,
  time_t timeout):
   TimeBasedCommand(cuid, e, 1),
-  _requestGroup(requestGroup),
-  _timeout(timeout)
+  requestGroup_(requestGroup),
+  timeout_(timeout)
 {}
 
 void BtStopDownloadCommand::preProcess()
 {
-  if(_btRuntime->isHalt() || _pieceStorage->downloadFinished()) {
-    _exit = true;
+  if(btRuntime_->isHalt() || pieceStorage_->downloadFinished()) {
+    enableExit();
   }
-  if(_checkPoint.elapsed(_timeout)) {
-    logger->notice("GID#%d Stop downloading torrent due to"
-                   " --bt-stop-timeout option.", _requestGroup->getGID());
-    _requestGroup->setHaltRequested(true);
-    _exit = true;
+  if(checkPoint_.difference(global::wallclock) >= timeout_) {
+    getLogger()->notice("GID#%s Stop downloading torrent due to"
+                        " --bt-stop-timeout option.",
+                        util::itos(requestGroup_->getGID()).c_str());
+    requestGroup_->setHaltRequested(true);
+    enableExit();
   }
 }
 
 void BtStopDownloadCommand::process()
 {
-  if(_requestGroup->calculateStat().getDownloadSpeed() > 0) {
-    _checkPoint.reset();
+  if(requestGroup_->calculateStat().getDownloadSpeed() > 0) {
+    checkPoint_ = global::wallclock;
   }
 }
 
