@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -46,7 +46,6 @@
 #include "DirectDiskAdaptor.h"
 #include "MultiDiskAdaptor.h"
 #include "DiskWriter.h"
-#include "BitfieldManFactory.h"
 #include "BitfieldMan.h"
 #include "message.h"
 #include "DefaultDiskWriterFactory.h"
@@ -59,65 +58,66 @@
 #include "RarestPieceSelector.h"
 #include "array_fun.h"
 #include "PieceStatMan.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
 (const SharedHandle<DownloadContext>& downloadContext, const Option* option):
-  downloadContext(downloadContext),
-  bitfieldMan(BitfieldManFactory::getFactoryInstance()->
-	      createBitfieldMan(downloadContext->getPieceLength(),
-				downloadContext->getTotalLength())),
-  _diskWriterFactory(new DefaultDiskWriterFactory()),
-  endGamePieceNum(END_GAME_PIECE_NUM),
-  logger(LogFactory::getInstance()),
-  option(option),
-  _pieceStatMan(new PieceStatMan(downloadContext->getNumPieces(), true)),
-  _pieceSelector(new RarestPieceSelector(_pieceStatMan))
+  downloadContext_(downloadContext),
+  bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
+                               downloadContext->getTotalLength())),
+  diskWriterFactory_(new DefaultDiskWriterFactory()),
+  endGamePieceNum_(END_GAME_PIECE_NUM),
+  logger_(LogFactory::getInstance()),
+  option_(option),
+  pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
+  pieceSelector_(new RarestPieceSelector(pieceStatMan_))
 {}
 
 DefaultPieceStorage::~DefaultPieceStorage() {
-  delete bitfieldMan;
+  delete bitfieldMan_;
 }
 
 bool DefaultPieceStorage::isEndGame()
 {
-  return bitfieldMan->countMissingBlock() <= endGamePieceNum;
+  return bitfieldMan_->countMissingBlock() <= endGamePieceNum_;
 }
 
 bool DefaultPieceStorage::getMissingPieceIndex(size_t& index,
-					       const unsigned char* bitfield,
-					       size_t length)
+                                               const unsigned char* bitfield,
+                                               size_t length)
 {
-  const size_t mislen = bitfieldMan->getBitfieldLength();
+  const size_t mislen = bitfieldMan_->getBitfieldLength();
   array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
   bool r;
   if(isEndGame()) {
-    r = bitfieldMan->getAllMissingIndexes(misbitfield, mislen,
-					  bitfield, length);
+    r = bitfieldMan_->getAllMissingIndexes(misbitfield, mislen,
+                                           bitfield, length);
   } else {
-    r = bitfieldMan->getAllMissingUnusedIndexes(misbitfield, mislen,
-						bitfield, length);
+    r = bitfieldMan_->getAllMissingUnusedIndexes(misbitfield, mislen,
+                                                 bitfield, length);
   }
   if(r) {
     // We assume indexes is sorted using comparator less.
-    return _pieceSelector->select(index, misbitfield,bitfieldMan->countBlock());
+    return
+      pieceSelector_->select(index, misbitfield,bitfieldMan_->countBlock());
   } else {
     return false;
   }
 }
 
-PieceHandle DefaultPieceStorage::checkOutPiece(size_t index)
+SharedHandle<Piece> DefaultPieceStorage::checkOutPiece(size_t index)
 {
-  bitfieldMan->setUseBit(index);
+  bitfieldMan_->setUseBit(index);
 
-  PieceHandle piece = findUsedPiece(index);
+  SharedHandle<Piece> piece = findUsedPiece(index);
   if(piece.isNull()) {
-    piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
+    piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-    piece->setHashAlgo(downloadContext->getPieceHashAlgo());
+    piece->setHashAlgo(downloadContext_->getPieceHashAlgo());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -132,38 +132,40 @@ PieceHandle DefaultPieceStorage::checkOutPiece(size_t index)
  * Newly instantiated piece is not added to usedPieces.
  * Because it is waste of memory and there is no chance to use them later.
  */
-PieceHandle DefaultPieceStorage::getPiece(size_t index)
+SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
 {
-  PieceHandle piece;
-  if(0 <= index && index <= bitfieldMan->getMaxIndex()) {
+  SharedHandle<Piece> piece;
+  if(0 <= index && index <= bitfieldMan_->getMaxIndex()) {
     piece = findUsedPiece(index);
     if(piece.isNull()) {
-      piece.reset(new Piece(index, bitfieldMan->getBlockLength(index)));
+      piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
       if(hasPiece(index)) {
-	piece->setAllBlock();
+        piece->setAllBlock();
       }
     }
   }
   return piece;
 }
 
-void DefaultPieceStorage::addUsedPiece(const PieceHandle& piece)
+void DefaultPieceStorage::addUsedPiece(const SharedHandle<Piece>& piece)
 {
   std::deque<SharedHandle<Piece> >::iterator i =
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
-  usedPieces.insert(i, piece);
-  logger->debug("usedPieces.size()=%lu",
-		static_cast<unsigned long>(usedPieces.size()));
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
+  usedPieces_.insert(i, piece);
+  if(logger_->debug()) {
+    logger_->debug("usedPieces_.size()=%lu",
+                   static_cast<unsigned long>(usedPieces_.size()));
+  }
 }
 
-PieceHandle DefaultPieceStorage::findUsedPiece(size_t index) const
+SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
 {
   SharedHandle<Piece> p(new Piece());
   p->setIndex(index);
 
   std::deque<SharedHandle<Piece> >::const_iterator i =
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), p);
-  if(i != usedPieces.end() && (*i) == p) {
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), p);
+  if(i != usedPieces_.end() && (*i) == p) {
     return *i;
   } else {
     p.reset(0);
@@ -190,13 +192,14 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
 
 #ifdef ENABLE_BITTORRENT
 
-bool DefaultPieceStorage::hasMissingPiece(const PeerHandle& peer)
+bool DefaultPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
 {
-  return bitfieldMan->hasMissingPiece(peer->getBitfield(),
-				      peer->getBitfieldLength());
+  return bitfieldMan_->hasMissingPiece(peer->getBitfield(),
+                                       peer->getBitfieldLength());
 }
 
-PieceHandle DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+SharedHandle<Piece>
+DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
 {
   return getMissingPiece(peer->getBitfield(), peer->getBitfieldLength());
 }
@@ -204,21 +207,21 @@ PieceHandle DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
 void DefaultPieceStorage::createFastIndexBitfield
 (BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
 {
-  for(std::deque<size_t>::const_iterator itr =
-	peer->getPeerAllowedIndexSet().begin();
-      itr != peer->getPeerAllowedIndexSet().end(); ++itr) {
-    if(!bitfieldMan->isBitSet(*itr) && peer->hasPiece(*itr)) {
+  for(std::vector<size_t>::const_iterator itr =
+        peer->getPeerAllowedIndexSet().begin(),
+        eoi = peer->getPeerAllowedIndexSet().end(); itr != eoi; ++itr) {
+    if(!bitfieldMan_->isBitSet(*itr) && peer->hasPiece(*itr)) {
       bitfield.setBit(*itr);
     }
   }
 }
 
-PieceHandle DefaultPieceStorage::getMissingFastPiece
+SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 (const SharedHandle<Peer>& peer)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
-    BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-			     bitfieldMan->getTotalLength());
+    BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                             bitfieldMan_->getTotalLength());
     createFastIndexBitfield(tempBitfield, peer);
     return getMissingPiece(tempBitfield);
   } else {
@@ -227,30 +230,28 @@ PieceHandle DefaultPieceStorage::getMissingFastPiece
 }
 
 static void unsetExcludedIndexes(BitfieldMan& bitfield,
-				 const std::deque<size_t>& excludedIndexes)
+                                 const std::vector<size_t>& excludedIndexes)
 {
-  for(std::deque<size_t>::const_iterator i = excludedIndexes.begin();
-      i != excludedIndexes.end(); ++i) {
-    bitfield.unsetBit(*i);
-  }
+  std::for_each(excludedIndexes.begin(), excludedIndexes.end(),
+                std::bind1st(std::mem_fun(&BitfieldMan::unsetBit), &bitfield));
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer, const std::deque<size_t>& excludedIndexes)
+(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
 {
-  BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-			   bitfieldMan->getTotalLength());
+  BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                           bitfieldMan_->getTotalLength());
   tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
   unsetExcludedIndexes(tempBitfield, excludedIndexes);
   return getMissingPiece(tempBitfield);
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer, const std::deque<size_t>& excludedIndexes)
+(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
-    BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
-			     bitfieldMan->getTotalLength());
+    BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
+                             bitfieldMan_->getTotalLength());
     createFastIndexBitfield(tempBitfield, peer);
     unsetExcludedIndexes(tempBitfield, excludedIndexes);
     return getMissingPiece(tempBitfield);
@@ -261,18 +262,24 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 
 #endif // ENABLE_BITTORRENT
 
-PieceHandle DefaultPieceStorage::getSparseMissingUnusedPiece
+bool DefaultPieceStorage::hasMissingUnusedPiece()
+{
+  size_t index;
+  return bitfieldMan_->getFirstMissingUnusedIndex(index);
+}
+
+SharedHandle<Piece> DefaultPieceStorage::getSparseMissingUnusedPiece
 (const unsigned char* ignoreBitfield, size_t length)
 {
   size_t index;
-  if(bitfieldMan->getSparseMissingUnusedIndex(index, ignoreBitfield, length)) {
+  if(bitfieldMan_->getSparseMissingUnusedIndex(index, ignoreBitfield, length)) {
     return checkOutPiece(index);
   } else {
     return SharedHandle<Piece>();
   }
 }
 
-PieceHandle DefaultPieceStorage::getMissingPiece(size_t index)
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece(size_t index)
 {
   if(hasPiece(index) || isPieceUsed(index)) {
     return SharedHandle<Piece>();
@@ -281,15 +288,15 @@ PieceHandle DefaultPieceStorage::getMissingPiece(size_t index)
   }
 }
 
-void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece)
+void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 {
   if(piece.isNull()) {
     return;
   }
   std::deque<SharedHandle<Piece> >::iterator i = 
-    std::lower_bound(usedPieces.begin(), usedPieces.end(), piece);
-  if(i != usedPieces.end() && (*i) == piece) {
-    usedPieces.erase(i);
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
+  if(i != usedPieces_.end() && (*i) == piece) {
+    usedPieces_.erase(i);
   }
 }
 
@@ -308,18 +315,18 @@ void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece)
 // }
 
 // size_t DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
-// 						       size_t delNum)
+//                                                     size_t delNum)
 // {
 //   size_t deleted = 0;
 //   for(Pieces::iterator itr = usedPieces.begin();
 //       itr != usedPieces.end() && deleted < delNum;) {
-//     PieceHandle& piece = *itr;
+//     SharedHandle<Piece>& piece = *itr;
 //     if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
 //        piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
 //       logger->info(MSG_DELETING_USED_PIECE,
-// 		    piece->getIndex(),
-// 		    (piece->countCompleteBlock()*100)/piece->countBlock(),
-// 		    fillRate);
+//                  piece->getIndex(),
+//                  (piece->countCompleteBlock()*100)/piece->countBlock(),
+//                  fillRate);
 //       itr = usedPieces.erase(itr);
 //       ++deleted;
 //     } else {
@@ -329,46 +336,46 @@ void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece)
 //   return deleted;
 // }
 
-void DefaultPieceStorage::completePiece(const PieceHandle& piece)
+void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
 {
   if(piece.isNull()) {
     return;
   }
   deleteUsedPiece(piece);
-//   if(!isEndGame()) {
-//     reduceUsedPieces(100);
-//   }
+  //   if(!isEndGame()) {
+  //     reduceUsedPieces(100);
+  //   }
   if(allDownloadFinished()) {
     return;
   }
-  bitfieldMan->setBit(piece->getIndex());
-  bitfieldMan->unsetUseBit(piece->getIndex());
+  bitfieldMan_->setBit(piece->getIndex());
+  bitfieldMan_->unsetUseBit(piece->getIndex());
   addPieceStats(piece->getIndex());
   if(downloadFinished()) {
-    downloadContext->resetDownloadStopTime();
+    downloadContext_->resetDownloadStopTime();
     if(isSelectiveDownloadingMode()) {
-      logger->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
+      logger_->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
       // message after user-specified files were downloaded.
       //finishSelectiveDownloadingMode();
     } else {
-      logger->info(MSG_DOWNLOAD_COMPLETED);
+      logger_->info(MSG_DOWNLOAD_COMPLETED);
     }
   }
 }
 
 bool DefaultPieceStorage::isSelectiveDownloadingMode()
 {
-  return bitfieldMan->isFilterEnabled();
+  return bitfieldMan_->isFilterEnabled();
 }
 
 // not unittested
-void DefaultPieceStorage::cancelPiece(const PieceHandle& piece)
+void DefaultPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
 {
   if(piece.isNull()) {
     return;
   }
-  bitfieldMan->unsetUseBit(piece->getIndex());
+  bitfieldMan_->unsetUseBit(piece->getIndex());
   if(!isEndGame()) {
     if(piece->getCompletedLength() == 0) {
       deleteUsedPiece(piece);
@@ -378,28 +385,28 @@ void DefaultPieceStorage::cancelPiece(const PieceHandle& piece)
 
 bool DefaultPieceStorage::hasPiece(size_t index)
 {
-  return bitfieldMan->isBitSet(index);
+  return bitfieldMan_->isBitSet(index);
 }
 
 bool DefaultPieceStorage::isPieceUsed(size_t index)
 {
-  return bitfieldMan->isUseBitSet(index);
+  return bitfieldMan_->isUseBitSet(index);
 }
 
 uint64_t DefaultPieceStorage::getTotalLength()
 {
-  return bitfieldMan->getTotalLength();
+  return bitfieldMan_->getTotalLength();
 }
 
 uint64_t DefaultPieceStorage::getFilteredTotalLength()
 {
-  return bitfieldMan->getFilteredTotalLength();
+  return bitfieldMan_->getFilteredTotalLength();
 }
 
 uint64_t DefaultPieceStorage::getCompletedLength()
 {
   uint64_t completedLength =
-    bitfieldMan->getCompletedLength()+getInFlightPieceCompletedLength();
+    bitfieldMan_->getCompletedLength()+getInFlightPieceCompletedLength();
   uint64_t totalLength = getTotalLength();
   if(completedLength > totalLength) {
     completedLength = totalLength;
@@ -409,22 +416,26 @@ uint64_t DefaultPieceStorage::getCompletedLength()
 
 uint64_t DefaultPieceStorage::getFilteredCompletedLength()
 {
-  return bitfieldMan->getFilteredCompletedLength()+getInFlightPieceCompletedLength();
+  return bitfieldMan_->getFilteredCompletedLength()+
+    getInFlightPieceCompletedLength();
 }
 
 size_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 {
-  return std::accumulate(usedPieces.begin(), usedPieces.end(), 0, adopt2nd(std::plus<size_t>(), mem_fun_sh(&Piece::getCompletedLength)));
+  return std::accumulate(usedPieces_.begin(), usedPieces_.end(),
+                         0, adopt2nd(std::plus<size_t>(),
+                                     mem_fun_sh(&Piece::getCompletedLength)));
 }
 
 // not unittested
 void DefaultPieceStorage::setupFileFilter()
 {
   const std::vector<SharedHandle<FileEntry> >& fileEntries =
-    downloadContext->getFileEntries();
+    downloadContext_->getFileEntries();
   bool allSelected = true;
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-	fileEntries.begin(); i != fileEntries.end(); ++i) {
+        fileEntries.begin(), eoi = fileEntries.end();
+      i != eoi; ++i) {
     if(!(*i)->isRequested()) {
       allSelected = false;
       break;
@@ -434,113 +445,121 @@ void DefaultPieceStorage::setupFileFilter()
     return;
   }
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-	fileEntries.begin(); i != fileEntries.end(); ++i) {
+        fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
     if((*i)->isRequested()) {
-      bitfieldMan->addFilter((*i)->getOffset(), (*i)->getLength());
+      bitfieldMan_->addFilter((*i)->getOffset(), (*i)->getLength());
     }
   }
-  bitfieldMan->enableFilter();
+  bitfieldMan_->enableFilter();
 }
 
 // not unittested
 void DefaultPieceStorage::clearFileFilter()
 {
-  bitfieldMan->clearFilter();
+  bitfieldMan_->clearFilter();
 }
 
 // not unittested
 bool DefaultPieceStorage::downloadFinished()
 {
-  // TODO iterate all requested FileEntry and Call bitfieldMan->isBitSetOffsetRange()
-  return bitfieldMan->isFilteredAllBitSet();
+  // TODO iterate all requested FileEntry and Call
+  // bitfieldMan->isBitSetOffsetRange()
+  return bitfieldMan_->isFilteredAllBitSet();
 }
 
 // not unittested
 bool DefaultPieceStorage::allDownloadFinished()
 {
-  return bitfieldMan->isAllBitSet();
+  return bitfieldMan_->isAllBitSet();
 }
 
 // not unittested
 void DefaultPieceStorage::initStorage()
 {
-  if(downloadContext->getFileEntries().size() == 1) {
-    logger->debug("Instantiating DirectDiskAdaptor");
+  if(downloadContext_->getFileEntries().size() == 1) {
+    if(logger_->debug()) {
+      logger_->debug("Instantiating DirectDiskAdaptor");
+    }
     DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
-    directDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
-    directDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
-				      downloadContext->getFileEntries().end());
+    directDiskAdaptor->setTotalLength(downloadContext_->getTotalLength());
+    directDiskAdaptor->setFileEntries
+      (downloadContext_->getFileEntries().begin(),
+       downloadContext_->getFileEntries().end());
 
     DiskWriterHandle writer =
-      _diskWriterFactory->newDiskWriter(directDiskAdaptor->getFilePath());
-    if(option->getAsBool(PREF_ENABLE_DIRECT_IO)) {
+      diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath());
+    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
       writer->allowDirectIO();
     }
 
     directDiskAdaptor->setDiskWriter(writer);
-    this->diskAdaptor = directDiskAdaptor;
+    diskAdaptor_ = directDiskAdaptor;
   } else {
-    logger->debug("Instantiating MultiDiskAdaptor");
+    if(logger_->debug()) {
+      logger_->debug("Instantiating MultiDiskAdaptor");
+    }
     MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
-    multiDiskAdaptor->setFileEntries(downloadContext->getFileEntries().begin(),
-				     downloadContext->getFileEntries().end());
-    if(option->getAsBool(PREF_ENABLE_DIRECT_IO)) {
+    multiDiskAdaptor->setFileEntries(downloadContext_->getFileEntries().begin(),
+                                     downloadContext_->getFileEntries().end());
+    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
       multiDiskAdaptor->allowDirectIO();
     }
-    multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
-    multiDiskAdaptor->setMaxOpenFiles(option->getAsInt(PREF_BT_MAX_OPEN_FILES));
-    this->diskAdaptor = multiDiskAdaptor;
+    multiDiskAdaptor->setPieceLength(downloadContext_->getPieceLength());
+    multiDiskAdaptor->setMaxOpenFiles
+      (option_->getAsInt(PREF_BT_MAX_OPEN_FILES));
+    diskAdaptor_ = multiDiskAdaptor;
   }
 #ifdef HAVE_POSIX_FALLOCATE
-  if(option->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
-    diskAdaptor->enableFallocate();
+  if(option_->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
+    diskAdaptor_->enableFallocate();
   }
 #endif // HAVE_POSIX_FALLOCATE
 }
 
 void DefaultPieceStorage::setBitfield(const unsigned char* bitfield,
-				      size_t bitfieldLength)
+                                      size_t bitfieldLength)
 {
-  bitfieldMan->setBitfield(bitfield, bitfieldLength);
+  bitfieldMan_->setBitfield(bitfield, bitfieldLength);
   addPieceStats(bitfield, bitfieldLength);
 }
 
 size_t DefaultPieceStorage::getBitfieldLength()
 {
-  return bitfieldMan->getBitfieldLength();
+  return bitfieldMan_->getBitfieldLength();
 }
 
 const unsigned char* DefaultPieceStorage::getBitfield()
 {
-  return bitfieldMan->getBitfield();
+  return bitfieldMan_->getBitfield();
 }
 
 DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
-  return diskAdaptor;
+  return diskAdaptor_;
 }
 
 size_t DefaultPieceStorage::getPieceLength(size_t index)
 {
-  return bitfieldMan->getBlockLength(index);
+  return bitfieldMan_->getBlockLength(index);
 }
 
-void DefaultPieceStorage::advertisePiece(int32_t cuid, size_t index)
+void DefaultPieceStorage::advertisePiece(cuid_t cuid, size_t index)
 {
-  HaveEntry entry(cuid, index);
-  haves.push_front(entry);
+  HaveEntry entry(cuid, index, global::wallclock);
+  haves_.push_front(entry);
 }
 
 void
-DefaultPieceStorage::getAdvertisedPieceIndexes(std::deque<size_t>& indexes,
-					       int32_t myCuid,
-					       const Time& lastCheckTime)
+DefaultPieceStorage::getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
+                                               cuid_t myCuid,
+                                               const Timer& lastCheckTime)
 {
-  for(Haves::const_iterator itr = haves.begin(); itr != haves.end(); itr++) {
-    const Haves::value_type& have = *itr;
+  for(std::deque<HaveEntry>::const_iterator itr = haves_.begin(),
+        eoi = haves_.end(); itr != eoi; ++itr) {
+    const HaveEntry& have = *itr;
     if(have.getCuid() == myCuid) {
       continue;
     }
-    if(lastCheckTime.isNewer(have.getRegisteredTime())) {
+    if(lastCheckTime > have.getRegisteredTime()) {
       break;
     }
     indexes.push_back(have.getIndex());
@@ -555,7 +574,7 @@ public:
   FindElapsedHave(time_t elapsed):elapsed(elapsed) {}
 
   bool operator()(const HaveEntry& have) {
-    if(have.getRegisteredTime().elapsed(elapsed)) {
+    if(have.getRegisteredTime().difference(global::wallclock) >= elapsed) {
       return true;
     } else {
       return false;
@@ -565,39 +584,46 @@ public:
   
 void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
 {
-  Haves::iterator itr =
-    std::find_if(haves.begin(), haves.end(), FindElapsedHave(elapsed));
-  if(itr != haves.end()) {
-    logger->debug(MSG_REMOVED_HAVE_ENTRY, haves.end()-itr);
-    haves.erase(itr, haves.end());
+  std::deque<HaveEntry>::iterator itr =
+    std::find_if(haves_.begin(), haves_.end(), FindElapsedHave(elapsed));
+  if(itr != haves_.end()) {
+    if(logger_->debug()) {
+      logger_->debug(MSG_REMOVED_HAVE_ENTRY, haves_.end()-itr);
+    }
+    haves_.erase(itr, haves_.end());
   }
 }
 
 void DefaultPieceStorage::markAllPiecesDone()
 {
-  bitfieldMan->setAllBit();
+  bitfieldMan_->setAllBit();
 }
 
 void DefaultPieceStorage::markPiecesDone(uint64_t length)
 {
-  if(length == bitfieldMan->getTotalLength()) {
-    bitfieldMan->setAllBit();
+  if(length == bitfieldMan_->getTotalLength()) {
+    bitfieldMan_->setAllBit();
+  } else if(length == 0) {
+    // TODO this would go to markAllPiecesUndone()
+    bitfieldMan_->clearAllBit();
+    usedPieces_.clear();
   } else {
-    size_t numPiece = length/bitfieldMan->getBlockLength();
+    size_t numPiece = length/bitfieldMan_->getBlockLength();
     if(numPiece > 0) {
-      bitfieldMan->setBitRange(0, numPiece-1);
+      bitfieldMan_->setBitRange(0, numPiece-1);
     }
-    size_t r = (length%bitfieldMan->getBlockLength())/Piece::BLOCK_LENGTH;
+    size_t r = (length%bitfieldMan_->getBlockLength())/Piece::BLOCK_LENGTH;
     if(r > 0) {
-      PieceHandle p(new Piece(numPiece, bitfieldMan->getBlockLength(numPiece)));
+      SharedHandle<Piece> p
+        (new Piece(numPiece, bitfieldMan_->getBlockLength(numPiece)));
       
       for(size_t i = 0; i < r; ++i) {
-	p->completeBlock(i);
+        p->completeBlock(i);
       }
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-      p->setHashAlgo(downloadContext->getPieceHashAlgo());
+      p->setHashAlgo(downloadContext_->getPieceHashAlgo());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -608,53 +634,56 @@ void DefaultPieceStorage::markPiecesDone(uint64_t length)
 
 void DefaultPieceStorage::markPieceMissing(size_t index)
 {
-  bitfieldMan->unsetBit(index);
+  bitfieldMan_->unsetBit(index);
 }
 
-void DefaultPieceStorage::addInFlightPiece(const Pieces& pieces)
+void DefaultPieceStorage::addInFlightPiece
+(const std::vector<SharedHandle<Piece> >& pieces)
 {
-  usedPieces.insert(usedPieces.end(), pieces.begin(), pieces.end());
-  std::sort(usedPieces.begin(), usedPieces.end());
+  usedPieces_.insert(usedPieces_.end(), pieces.begin(), pieces.end());
+  std::sort(usedPieces_.begin(), usedPieces_.end());
 }
 
 size_t DefaultPieceStorage::countInFlightPiece()
 {
-  return usedPieces.size();
+  return usedPieces_.size();
 }
 
-void DefaultPieceStorage::getInFlightPieces(std::deque<SharedHandle<Piece> >& pieces)
+void DefaultPieceStorage::getInFlightPieces
+(std::vector<SharedHandle<Piece> >& pieces)
 {
-  pieces.insert(pieces.end(), usedPieces.begin(), usedPieces.end());
+  pieces.insert(pieces.end(), usedPieces_.begin(), usedPieces_.end());
 }
 
-void DefaultPieceStorage::setDiskWriterFactory(const DiskWriterFactoryHandle& diskWriterFactory)
+void DefaultPieceStorage::setDiskWriterFactory
+(const DiskWriterFactoryHandle& diskWriterFactory)
 {
-  _diskWriterFactory = diskWriterFactory;
+  diskWriterFactory_ = diskWriterFactory;
 }
 
 void DefaultPieceStorage::addPieceStats(const unsigned char* bitfield,
-					size_t bitfieldLength)
+                                        size_t bitfieldLength)
 {
-  _pieceStatMan->addPieceStats(bitfield, bitfieldLength);
+  pieceStatMan_->addPieceStats(bitfield, bitfieldLength);
 }
 
 void DefaultPieceStorage::subtractPieceStats(const unsigned char* bitfield,
-					     size_t bitfieldLength)
+                                             size_t bitfieldLength)
 {
-  _pieceStatMan->subtractPieceStats(bitfield, bitfieldLength);
+  pieceStatMan_->subtractPieceStats(bitfield, bitfieldLength);
 }
 
 void DefaultPieceStorage::updatePieceStats(const unsigned char* newBitfield,
-					   size_t newBitfieldLength,
-					   const unsigned char* oldBitfield)
+                                           size_t newBitfieldLength,
+                                           const unsigned char* oldBitfield)
 {
-  _pieceStatMan->updatePieceStats(newBitfield, newBitfieldLength,
-				   oldBitfield);
+  pieceStatMan_->updatePieceStats(newBitfield, newBitfieldLength,
+                                  oldBitfield);
 }
 
 void DefaultPieceStorage::addPieceStats(size_t index)
 {
-  _pieceStatMan->addPieceStats(index);
+  pieceStatMan_->addPieceStats(index);
 }
 
 } // namespace aria2
