@@ -54,38 +54,38 @@ class PieceSelector;
 
 class HaveEntry {
 private:
-  cuid_t cuid;
-  size_t index;
-  Timer registeredTime;
+  cuid_t cuid_;
+  size_t index_;
+  Timer registeredTime_;
 public:
   HaveEntry(cuid_t cuid, size_t index, const Timer& registeredTime):
-    cuid(cuid),
-    index(index),
-    registeredTime(registeredTime) {}
+    cuid_(cuid),
+    index_(index),
+    registeredTime_(registeredTime) {}
 
-  cuid_t getCuid() const { return cuid; }
+  cuid_t getCuid() const { return cuid_; }
 
-  size_t getIndex() const { return index; }
+  size_t getIndex() const { return index_; }
 
-  const Timer& getRegisteredTime() const { return registeredTime; }
+  const Timer& getRegisteredTime() const { return registeredTime_; }
 };
 
 class DefaultPieceStorage : public PieceStorage {
 private:
-  SharedHandle<DownloadContext> downloadContext;
-  BitfieldMan* bitfieldMan;
-  SharedHandle<DiskAdaptor> diskAdaptor;
-  SharedHandle<DiskWriterFactory> _diskWriterFactory;
-  std::deque<SharedHandle<Piece> > usedPieces;
+  SharedHandle<DownloadContext> downloadContext_;
+  BitfieldMan* bitfieldMan_;
+  SharedHandle<DiskAdaptor> diskAdaptor_;
+  SharedHandle<DiskWriterFactory> diskWriterFactory_;
+  std::deque<SharedHandle<Piece> > usedPieces_;
 
-  size_t endGamePieceNum;
-  Logger* logger;
-  const Option* option;
-  std::deque<HaveEntry> haves;
+  size_t endGamePieceNum_;
+  Logger* logger_;
+  const Option* option_;
+  std::deque<HaveEntry> haves_;
 
-  SharedHandle<PieceStatMan> _pieceStatMan;
+  SharedHandle<PieceStatMan> pieceStatMan_;
 
-  SharedHandle<PieceSelector> _pieceSelector;
+  SharedHandle<PieceSelector> pieceSelector_;
 
   bool getMissingPieceIndex(size_t& index,
                             const unsigned char* bitfield, size_t length);
@@ -124,7 +124,8 @@ public:
 
   virtual SharedHandle<Piece> getMissingPiece(const SharedHandle<Peer>& peer);
 
-  virtual SharedHandle<Piece> getMissingFastPiece(const SharedHandle<Peer>& peer);
+  virtual SharedHandle<Piece> getMissingFastPiece
+  (const SharedHandle<Peer>& peer);
 
   virtual SharedHandle<Piece> getMissingPiece
   (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes);
@@ -177,11 +178,11 @@ public:
   virtual const unsigned char* getBitfield();
 
   virtual void setEndGamePieceNum(size_t num) {
-    endGamePieceNum = num;
+    endGamePieceNum_ = num;
   }
 
   size_t getEndGamePieceNum() const {
-    return endGamePieceNum;
+    return endGamePieceNum_;
   }
 
   virtual bool isSelectiveDownloadingMode();
@@ -231,21 +232,22 @@ public:
    */
   void addUsedPiece(const SharedHandle<Piece>& piece);
 
-  void setDiskWriterFactory(const SharedHandle<DiskWriterFactory>& diskWriterFactory);
+  void setDiskWriterFactory
+  (const SharedHandle<DiskWriterFactory>& diskWriterFactory);
 
   const SharedHandle<PieceStatMan>& getPieceStatMan() const
   {
-    return _pieceStatMan;
+    return pieceStatMan_;
   }
 
   void setPieceSelector(const SharedHandle<PieceSelector>& pieceSelector)
   {
-    _pieceSelector = pieceSelector;
+    pieceSelector_ = pieceSelector;
   }
 
   const SharedHandle<PieceSelector>& getPieceSelector() const
   {
-    return _pieceSelector;
+    return pieceSelector_;
   }
     
 };
