@@ -59,6 +59,8 @@
 #include "Decoder.h"
 #include "RequestGroupMan.h"
 #include "wallclock.h"
+#include "ServerStatMan.h"
+#include "FileAllocationEntry.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigestHelper.h"
 #endif // ENABLE_MESSAGE_DIGEST
@@ -67,90 +69,96 @@ namespace aria2 {
 
 static const size_t BUFSIZE = 16*1024;
 
-DownloadCommand::DownloadCommand(int cuid,
+DownloadCommand::DownloadCommand(cuid_t cuid,
                                  const SharedHandle<Request>& req,
                                  const SharedHandle<FileEntry>& fileEntry,
                                  RequestGroup* requestGroup,
                                  DownloadEngine* e,
                                  const SocketHandle& s):
   AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
-  _buf(new unsigned char[BUFSIZE])
+  buf_(new unsigned char[BUFSIZE]),
+  startupIdleTime_(10),
+  lowestDownloadSpeedLimit_(0)
 #ifdef ENABLE_MESSAGE_DIGEST
-  , _pieceHashValidationEnabled(false)
+  , pieceHashValidationEnabled_(false)
 #endif // ENABLE_MESSAGE_DIGEST
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   {
     if(getOption()->getAsBool(PREF_REALTIME_CHUNK_CHECKSUM)) {
-      std::string algo = _requestGroup->getDownloadContext()->getPieceHashAlgo();
+      const std::string& algo = getDownloadContext()->getPieceHashAlgo();
       if(MessageDigestContext::supports(algo)) {
-        _messageDigestContext.reset(new MessageDigestContext());
-        _messageDigestContext->trySetAlgo(algo);
-        _messageDigestContext->digestInit();
+        messageDigestContext_.reset(new MessageDigestContext());
+        messageDigestContext_->trySetAlgo(algo);
+        messageDigestContext_->digestInit();
         
-        _pieceHashValidationEnabled = true;
+        pieceHashValidationEnabled_ = true;
       }
     }
   }
 #endif // ENABLE_MESSAGE_DIGEST
 
-  peerStat = req->initPeerStat();
-  peerStat->downloadStart();
-  _requestGroup->getSegmentMan()->registerPeerStat(peerStat);
+  peerStat_ = req->initPeerStat();
+  peerStat_->downloadStart();
+  getSegmentMan()->registerPeerStat(peerStat_);
 }
 
 DownloadCommand::~DownloadCommand() {
-  peerStat->downloadStop();
-  _requestGroup->getSegmentMan()->updateFastestPeerStat(peerStat);
-  delete [] _buf;
+  peerStat_->downloadStop();
+  getSegmentMan()->updateFastestPeerStat(peerStat_);
+  delete [] buf_;
 }
 
 bool DownloadCommand::executeInternal() {
-  if(e->_requestGroupMan->doesOverallDownloadSpeedExceed() ||
-     _requestGroup->doesDownloadSpeedExceed()) {
-    e->commands.push_back(this);
+  if(getDownloadEngine()->getRequestGroupMan()->doesOverallDownloadSpeedExceed()
+     || getRequestGroup()->doesDownloadSpeedExceed()) {
+    getDownloadEngine()->addCommand(this);
     disableReadCheckSocket();
     return false;
   }
-  setReadCheckSocket(socket);
-  SharedHandle<Segment> segment = _segments.front();
+  setReadCheckSocket(getSocket());
+  SharedHandle<Segment> segment = getSegments().front();
 
   size_t bufSize;
   if(segment->getLength() > 0) {
     if(static_cast<uint64_t>(segment->getPosition()+segment->getLength()) <=
-       static_cast<uint64_t>(_fileEntry->getLastOffset())) {
+       static_cast<uint64_t>(getFileEntry()->getLastOffset())) {
       bufSize = std::min(segment->getLength()-segment->getWrittenLength(),
                          BUFSIZE);
     } else {
-      bufSize = std::min(static_cast<size_t>(_fileEntry->getLastOffset()-segment->getPositionToWrite()), BUFSIZE);
+      bufSize =
+        std::min
+        (static_cast<size_t>
+         (getFileEntry()->getLastOffset()-segment->getPositionToWrite()),
+         BUFSIZE);
     }
   } else {
     bufSize = BUFSIZE;
   }
-  socket->readData(_buf, bufSize);
+  getSocket()->readData(buf_, bufSize);
 
   const SharedHandle<DiskAdaptor>& diskAdaptor =
-    _requestGroup->getPieceStorage()->getDiskAdaptor();
+    getPieceStorage()->getDiskAdaptor();
 
   const unsigned char* bufFinal;
   size_t bufSizeFinal;
 
   std::string decoded;
-  if(_transferEncodingDecoder.isNull()) {
-    bufFinal = _buf;
+  if(transferEncodingDecoder_.isNull()) {
+    bufFinal = buf_;
     bufSizeFinal = bufSize;
   } else {
-    decoded = _transferEncodingDecoder->decode(_buf, bufSize);
+    decoded = transferEncodingDecoder_->decode(buf_, bufSize);
 
     bufFinal = reinterpret_cast<const unsigned char*>(decoded.c_str());
     bufSizeFinal = decoded.size();
   }
 
-  if(_contentEncodingDecoder.isNull()) {
+  if(contentEncodingDecoder_.isNull()) {
     diskAdaptor->writeData(bufFinal, bufSizeFinal,
                            segment->getPositionToWrite());
   } else {
-    std::string out = _contentEncodingDecoder->decode(bufFinal, bufSizeFinal);
+    std::string out = contentEncodingDecoder_->decode(bufFinal, bufSizeFinal);
     diskAdaptor->writeData(reinterpret_cast<const unsigned char*>(out.data()),
                            out.size(),
                            segment->getPositionToWrite());
@@ -159,7 +167,7 @@ bool DownloadCommand::executeInternal() {
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-  if(_pieceHashValidationEnabled) {
+  if(pieceHashValidationEnabled_) {
     segment->updateHash(segment->getWrittenLength(), bufFinal, bufSizeFinal);
   }
 
@@ -167,35 +175,33 @@ bool DownloadCommand::executeInternal() {
   if(bufSizeFinal > 0) {
     segment->updateWrittenLength(bufSizeFinal);
   }
-
-  peerStat->updateDownloadLength(bufSize);
-
-  _requestGroup->getSegmentMan()->updateDownloadSpeedFor(peerStat);
-
+  peerStat_->updateDownloadLength(bufSize);
+  getSegmentMan()->updateDownloadSpeedFor(peerStat_);
   bool segmentPartComplete = false;
   // Note that GrowSegment::complete() always returns false.
-  if(_transferEncodingDecoder.isNull() && _contentEncodingDecoder.isNull()) {
+  if(transferEncodingDecoder_.isNull() && contentEncodingDecoder_.isNull()) {
     if(segment->complete() ||
-       segment->getPositionToWrite() == _fileEntry->getLastOffset()) {
+       segment->getPositionToWrite() == getFileEntry()->getLastOffset()) {
       segmentPartComplete = true;
     } else if(segment->getLength() == 0 && bufSize == 0 &&
-              !socket->wantRead() && !socket->wantWrite()) {
+              !getSocket()->wantRead() && !getSocket()->wantWrite()) {
       segmentPartComplete = true;
     }
-  } else if(!_transferEncodingDecoder.isNull() &&
-            (segment->complete() || segment->getPositionToWrite() == _fileEntry->getLastOffset())) {
+  } else if(!transferEncodingDecoder_.isNull() &&
+            (segment->complete() ||
+             segment->getPositionToWrite() == getFileEntry()->getLastOffset())){
     // In this case, transferEncodingDecoder is used and
     // Content-Length is known.
     segmentPartComplete = true;
-  } else if((_transferEncodingDecoder.isNull() ||
-             _transferEncodingDecoder->finished()) &&
-            (_contentEncodingDecoder.isNull() ||
-             _contentEncodingDecoder->finished())) {
+  } else if((transferEncodingDecoder_.isNull() ||
+             transferEncodingDecoder_->finished()) &&
+            (contentEncodingDecoder_.isNull() ||
+             contentEncodingDecoder_->finished())) {
     segmentPartComplete = true;
   }
 
   if(!segmentPartComplete && bufSize == 0 &&
-     !socket->wantRead() && !socket->wantWrite()) {
+     !getSocket()->wantRead() && !getSocket()->wantWrite()) {
     throw DL_RETRY_EX(EX_GOT_EOF);
   }
 
@@ -204,49 +210,55 @@ bool DownloadCommand::executeInternal() {
       // If segment->getLength() == 0, the server doesn't provide
       // content length, but the client detected that download
       // completed.
-      logger->info(MSG_SEGMENT_DOWNLOAD_COMPLETED, cuid);
+      if(getLogger()->info()) {
+        getLogger()->info(MSG_SEGMENT_DOWNLOAD_COMPLETED,
+                          util::itos(getCuid()).c_str());
+      }
 #ifdef ENABLE_MESSAGE_DIGEST
 
       {
-        std::string expectedPieceHash =
-          _requestGroup->getDownloadContext()->getPieceHash(segment->getIndex());
-        if(_pieceHashValidationEnabled && !expectedPieceHash.empty()) {
+        const std::string& expectedPieceHash =
+          getDownloadContext()->getPieceHash(segment->getIndex());
+        if(pieceHashValidationEnabled_ && !expectedPieceHash.empty()) {
           if(segment->isHashCalculated()) {
-            if(logger->debug()) {
-              logger->debug("Hash is available! index=%lu",
-                            static_cast<unsigned long>(segment->getIndex()));
+            if(getLogger()->debug()) {
+              getLogger()->debug
+                ("Hash is available! index=%lu",
+                 static_cast<unsigned long>(segment->getIndex()));
             }
-            validatePieceHash(segment, expectedPieceHash, segment->getHashString());
+            validatePieceHash
+              (segment, expectedPieceHash, segment->getHashString());
           } else {
-            _messageDigestContext->digestReset();
-            validatePieceHash(segment, expectedPieceHash,
-                              MessageDigestHelper::digest
-                              (_messageDigestContext.get(),
-                               _requestGroup->getPieceStorage()->getDiskAdaptor(),
-                               segment->getPosition(),
-                               segment->getLength()));
+            messageDigestContext_->digestReset();
+            validatePieceHash
+              (segment, expectedPieceHash,
+               MessageDigestHelper::digest
+               (messageDigestContext_.get(),
+                getPieceStorage()->getDiskAdaptor(),
+                segment->getPosition(),
+                segment->getLength()));
           }
         } else {
-          _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
+          getSegmentMan()->completeSegment(getCuid(), segment);
         }
       }
 
 #else // !ENABLE_MESSAGE_DIGEST
-      _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
+      getSegmentMan()->completeSegment(getCuid(), segment);
 #endif // !ENABLE_MESSAGE_DIGEST
     } else {
       // If segment is not canceled here, in the next pipelining
       // request, aria2 requests bad range
       // [FileEntry->getLastOffset(), FileEntry->getLastOffset())
-      _requestGroup->getSegmentMan()->cancelSegment(cuid, segment);
+      getSegmentMan()->cancelSegment(getCuid(), segment);
     }
     checkLowestDownloadSpeed();
     // this unit is going to download another segment.
     return prepareForNextSegment();
   } else {
     checkLowestDownloadSpeed();
-    setWriteCheckSocketIf(socket, socket->wantWrite());
-    e->commands.push_back(this);
+    setWriteCheckSocketIf(getSocket(), getSocket()->wantWrite());
+    getDownloadEngine()->addCommand(this);
     return false;
   }
 }
@@ -254,62 +266,69 @@ bool DownloadCommand::executeInternal() {
 void DownloadCommand::checkLowestDownloadSpeed() const
 {
   // calculate downloading speed
-  if(peerStat->getDownloadStartTime().difference(global::wallclock) >=
-     startupIdleTime) {
-    unsigned int nowSpeed = peerStat->calculateDownloadSpeed();
-    if(lowestDownloadSpeedLimit > 0 &&  nowSpeed <= lowestDownloadSpeedLimit) {
+  if(peerStat_->getDownloadStartTime().difference(global::wallclock) >=
+     startupIdleTime_) {
+    unsigned int nowSpeed = peerStat_->calculateDownloadSpeed();
+    if(lowestDownloadSpeedLimit_ > 0 && nowSpeed <= lowestDownloadSpeedLimit_) {
       throw DL_ABORT_EX2(StringFormat(EX_TOO_SLOW_DOWNLOAD_SPEED,
                                       nowSpeed,
-                                      lowestDownloadSpeedLimit,
-                                      req->getHost().c_str()).str(),
+                                      lowestDownloadSpeedLimit_,
+                                      getRequest()->getHost().c_str()).str(),
                          downloadresultcode::TOO_SLOW_DOWNLOAD_SPEED);
     }
   }
 }
 
 bool DownloadCommand::prepareForNextSegment() {
-  if(_requestGroup->downloadFinished()) {
+  if(getRequestGroup()->downloadFinished()) {
     // Remove in-flight request here.
-    _fileEntry->poolRequest(req);
+    getFileEntry()->poolRequest(getRequest());
     // If this is a single file download, and file size becomes known
     // just after downloading, set total length to FileEntry object
     // here.
-    if(_requestGroup->getDownloadContext()->getFileEntries().size() == 1) {
-      const SharedHandle<FileEntry>& fileEntry =
-        _requestGroup->getDownloadContext()->getFirstFileEntry();
-      if(fileEntry->getLength() == 0) {
-        fileEntry->setLength
-          (_requestGroup->getPieceStorage()->getCompletedLength());
+    if(getDownloadContext()->getFileEntries().size() == 1) {
+      if(getFileEntry()->getLength() == 0) {
+        getFileEntry()->setLength(getPieceStorage()->getCompletedLength());
       }
     }
 #ifdef ENABLE_MESSAGE_DIGEST
-    SharedHandle<CheckIntegrityEntry> entry
-      (new ChecksumCheckIntegrityEntry(_requestGroup));
-    if(entry->isValidationReady()) {
-      entry->initValidator();
-      // TODO do we need cuttrailinggarbage here?
-      e->_checkIntegrityMan->pushEntry(entry);
+    if(getDownloadContext()->getPieceHashAlgo().empty()) {
+      SharedHandle<CheckIntegrityEntry> entry
+        (new ChecksumCheckIntegrityEntry(getRequestGroup()));
+      if(entry->isValidationReady()) {
+        entry->initValidator();
+        entry->cutTrailingGarbage();
+        getDownloadEngine()->getCheckIntegrityMan()->pushEntry(entry);
+      }
     }
-    e->setNoWait(true);
-    e->setRefreshInterval(0);
+    // Following 2lines are needed for DownloadEngine to detect
+    // completed RequestGroups without 1sec delay.
+    getDownloadEngine()->setNoWait(true);
+    getDownloadEngine()->setRefreshInterval(0);
 #endif // ENABLE_MESSAGE_DIGEST
     return true;
   } else {
     // The number of segments should be 1 in order to pass through the next
     // segment.
-    if(_segments.size() == 1) {
-      SharedHandle<Segment> tempSegment = _segments.front();
+    if(getSegments().size() == 1) {
+      SharedHandle<Segment> tempSegment = getSegments().front();
       if(!tempSegment->complete()) {
         return prepareForRetry(0);
       }
-      SharedHandle<Segment> nextSegment =
-        _requestGroup->getSegmentMan()->getSegment(cuid,
-                                                   tempSegment->getIndex()+1);
-      if(!nextSegment.isNull() && nextSegment->getWrittenLength() == 0) {
-        e->commands.push_back(this);
-        return false;
-      } else {
+      SharedHandle<Segment> nextSegment = getSegmentMan()->getSegment
+        (getCuid(), tempSegment->getIndex()+1);
+      if(nextSegment.isNull()) {
+        nextSegment = getSegmentMan()->getCleanSegmentIfOwnerIsIdle
+        (getCuid(), tempSegment->getIndex()+1);
+      }
+      if(nextSegment.isNull() || nextSegment->getWrittenLength() > 0) {
+        // If nextSegment->getWrittenLength() > 0, current socket must
+        // be closed because writing incoming data at
+        // nextSegment->getWrittenLength() corrupts file.
         return prepareForRetry(0);
+      } else {
+        getDownloadEngine()->addCommand(this);
+        return false;
       }
     } else {
       return prepareForRetry(0);
@@ -324,16 +343,16 @@ void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
                                         const std::string& actualPieceHash)
 {
   if(actualPieceHash == expectedPieceHash) {
-    logger->info(MSG_GOOD_CHUNK_CHECKSUM, actualPieceHash.c_str());
-    _requestGroup->getSegmentMan()->completeSegment(cuid, segment);
+    getLogger()->info(MSG_GOOD_CHUNK_CHECKSUM, actualPieceHash.c_str());
+    getSegmentMan()->completeSegment(getCuid(), segment);
   } else {
-    logger->info(EX_INVALID_CHUNK_CHECKSUM,
-                 segment->getIndex(),
-                 util::itos(segment->getPosition(), true).c_str(),
-                 expectedPieceHash.c_str(),
-                 actualPieceHash.c_str());
+    getLogger()->info(EX_INVALID_CHUNK_CHECKSUM,
+                      segment->getIndex(),
+                      util::itos(segment->getPosition(), true).c_str(),
+                      expectedPieceHash.c_str(),
+                      actualPieceHash.c_str());
     segment->clear();
-    _requestGroup->getSegmentMan()->cancelSegment(cuid);
+    getSegmentMan()->cancelSegment(getCuid());
     throw DL_RETRY_EX
       (StringFormat("Invalid checksum index=%d", segment->getIndex()).str());
   }
@@ -344,13 +363,13 @@ void DownloadCommand::validatePieceHash(const SharedHandle<Segment>& segment,
 void DownloadCommand::setTransferEncodingDecoder
 (const SharedHandle<Decoder>& decoder)
 {
-  this->_transferEncodingDecoder = decoder;
+  this->transferEncodingDecoder_ = decoder;
 }
 
 void DownloadCommand::setContentEncodingDecoder
 (const SharedHandle<Decoder>& decoder)
 {
-  _contentEncodingDecoder = decoder;
+  contentEncodingDecoder_ = decoder;
 }
 
 } // namespace aria2
