@@ -48,11 +48,15 @@
 #include "RequestGroup.h"
 #include "Logger.h"
 #include "DownloadContext.h"
+#include "RequestGroupMan.h"
+#include "FileAllocationEntry.h"
+#include "CheckIntegrityEntry.h"
+#include "ServerStatMan.h"
 
 namespace aria2 {
 
 FtpFinishDownloadCommand::FtpFinishDownloadCommand
-(int cuid,
+(cuid_t cuid,
  const SharedHandle<Request>& req,
  const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
@@ -60,27 +64,20 @@ FtpFinishDownloadCommand::FtpFinishDownloadCommand
  DownloadEngine* e,
  const SharedHandle<SocketCore>& socket)
   :AbstractCommand(cuid, req, fileEntry, requestGroup, e, socket),
-   _ftpConnection(ftpConnection)
-{
-  e->addSocketForReadCheck(socket, this);
-}
-
-FtpFinishDownloadCommand::~FtpFinishDownloadCommand()
-{
-  e->deleteSocketForReadCheck(socket, this);
-}
+   ftpConnection_(ftpConnection)
+{}
 
 // overrides AbstractCommand::execute().
 // AbstractCommand::_segments is empty.
 bool FtpFinishDownloadCommand::execute()
 {
-  if(_requestGroup->isHaltRequested()) {
+  if(getRequestGroup()->isHaltRequested()) {
     return true;
   }
   try {
-    unsigned int status = _ftpConnection->receiveResponse();
+    unsigned int status = ftpConnection_->receiveResponse();
     if(status == 0) {
-      e->commands.push_back(this);
+      getDownloadEngine()->addCommand(this);
       return false;
     }
     if(status != 226) {
@@ -88,13 +85,15 @@ bool FtpFinishDownloadCommand::execute()
     }
     if(getOption()->getAsBool(PREF_FTP_REUSE_CONNECTION)) {
       std::map<std::string, std::string> options;
-      options["baseWorkingDir"] = _ftpConnection->getBaseWorkingDir();
-      e->poolSocket(req, isProxyDefined(), socket, options);
+      options["baseWorkingDir"] = ftpConnection_->getBaseWorkingDir();
+      getDownloadEngine()->poolSocket
+        (getRequest(), ftpConnection_->getUser(), createProxyRequest(),
+         getSocket(), options);
     }
   } catch(RecoverableException& e) {
-    logger->info(EX_EXCEPTION_CAUGHT, e);
+    getLogger()->info(EX_EXCEPTION_CAUGHT, e);
   }
-  if(_requestGroup->downloadFinished()) {
+  if(getRequestGroup()->downloadFinished()) {
     return true;
   } else {
     return prepareForRetry(0);
