@@ -40,6 +40,11 @@
 #include "RecoverableException.h"
 #include "SocketCore.h"
 #include "util.h"
+#include "RequestGroupMan.h"
+#include "FileAllocationEntry.h"
+#include "CheckIntegrityEntry.h"
+#include "ServerStatMan.h"
+#include "FileEntry.h"
 
 namespace aria2 {
 
@@ -48,40 +53,40 @@ LpdDispatchMessageCommand::LpdDispatchMessageCommand
  const SharedHandle<LpdMessageDispatcher>& dispatcher,
  DownloadEngine* e):
   Command(cuid),
-  _dispatcher(dispatcher),
-  _e(e),
-  _tryCount(0) {}
+  dispatcher_(dispatcher),
+  e_(e),
+  tryCount_(0) {}
 
 bool LpdDispatchMessageCommand::execute()
 {
-  if(_btRuntime->isHalt()) {
+  if(btRuntime_->isHalt()) {
     return true;
   }
-  if(_dispatcher->isAnnounceReady()) {
+  if(dispatcher_->isAnnounceReady()) {
     try {
-      logger->info("Dispatching LPD message for infohash=%s",
-                   util::toHex(_dispatcher->getInfoHash()).c_str());
-      if(_dispatcher->sendMessage()) {
-        logger->info("Sending LPD message is complete.");
-        _dispatcher->resetAnnounceTimer();
-        _tryCount = 0;
+      getLogger()->info("Dispatching LPD message for infohash=%s",
+                   util::toHex(dispatcher_->getInfoHash()).c_str());
+      if(dispatcher_->sendMessage()) {
+        getLogger()->info("Sending LPD message is complete.");
+        dispatcher_->resetAnnounceTimer();
+        tryCount_ = 0;
       } else {
-        ++_tryCount;
-        if(_tryCount >= 5) {
-          logger->info("Sending LPD message %u times but all failed.");
-          _dispatcher->resetAnnounceTimer();
-          _tryCount = 0;
+        ++tryCount_;
+        if(tryCount_ >= 5) {
+          getLogger()->info("Sending LPD message %u times but all failed.");
+          dispatcher_->resetAnnounceTimer();
+          tryCount_ = 0;
         } else {
-          logger->info("Could not send LPD message, retry shortly.");
+          getLogger()->info("Could not send LPD message, retry shortly.");
         }
       }
     } catch(RecoverableException& e) {
-      logger->info("Failed to send LPD message.", e);
-      _dispatcher->resetAnnounceTimer();
-      _tryCount = 0;
+      getLogger()->info("Failed to send LPD message.", e);
+      dispatcher_->resetAnnounceTimer();
+      tryCount_ = 0;
     }
   }
-  _e->commands.push_back(this);
+  e_->addCommand(this);
   return false;
 }
 
