@@ -67,7 +67,7 @@
 namespace aria2 {
 
 Metalink2RequestGroup::Metalink2RequestGroup():
-  _logger(LogFactory::getInstance()) {}
+  logger_(LogFactory::getInstance()) {}
 
 class AccumulateNonP2PUri {
 private:
@@ -113,7 +113,7 @@ Metalink2RequestGroup::generate
   std::vector<SharedHandle<RequestGroup> > tempgroups;
   createRequestGroup(tempgroups, entries, option);
   SharedHandle<MetadataInfo> mi;
-  if(metalinkFile == "-") {
+  if(metalinkFile == DEV_STDIN) {
     mi.reset(new MetadataInfo());
   } else {
     mi.reset(new MetadataInfo(metalinkFile));
@@ -144,7 +144,7 @@ Metalink2RequestGroup::createRequestGroup
  const SharedHandle<Option>& option)
 {
   if(entries.empty()) {
-    _logger->notice(EX_NO_RESULT_WITH_YOUR_PREFS);
+    logger_->notice(EX_NO_RESULT_WITH_YOUR_PREFS);
     return;
   }
   std::vector<int32_t> selectIndexes =
@@ -193,7 +193,7 @@ Metalink2RequestGroup::createRequestGroup
         entryGroups.begin(), eoi = entryGroups.end(); itr != eoi; ++itr) {
     const std::string& metaurl = (*itr).first;
     const std::vector<SharedHandle<MetalinkEntry> >& mes = (*itr).second;
-    _logger->info("Processing metaurl group metaurl=%s", metaurl.c_str());
+    logger_->info("Processing metaurl group metaurl=%s", metaurl.c_str());
 #ifdef ENABLE_BITTORRENT
     SharedHandle<RequestGroup> torrentRg;
     if(!metaurl.empty()) {
@@ -229,7 +229,7 @@ Metalink2RequestGroup::createRequestGroup
     SharedHandle<DownloadContext> dctx;
     if(mes.size() == 1) {
       SharedHandle<MetalinkEntry> entry = mes[0];
-      _logger->info(MSG_METALINK_QUEUEING, entry->getPath().c_str());
+      logger_->info(MSG_METALINK_QUEUEING, entry->getPath().c_str());
       entry->reorderResourcesByPriority();
       std::vector<std::string> uris;
       std::for_each(entry->resources.begin(), entry->resources.end(),
@@ -256,12 +256,11 @@ Metalink2RequestGroup::createRequestGroup
         dctx->getFirstFileEntry()->disableSingleHostMultiConnection();
       }
 #ifdef ENABLE_MESSAGE_DIGEST
-      if(entry->chunkChecksum.isNull()) {
-        if(!entry->checksum.isNull()) {
-          dctx->setChecksum(entry->checksum->getMessageDigest());
-          dctx->setChecksumHashAlgo(entry->checksum->getAlgo());
-        }
-      } else {
+      if(!entry->checksum.isNull()) {
+        dctx->setChecksum(entry->checksum->getMessageDigest());
+        dctx->setChecksumHashAlgo(entry->checksum->getAlgo());
+      }
+      if(!entry->chunkChecksum.isNull()) {
         dctx->setPieceHashes(entry->chunkChecksum->getChecksums().begin(),
                              entry->chunkChecksum->getChecksums().end());
         dctx->setPieceHashAlgo(entry->chunkChecksum->getAlgo());
@@ -281,9 +280,9 @@ Metalink2RequestGroup::createRequestGroup
       off_t offset = 0;
       for(std::vector<SharedHandle<MetalinkEntry> >::const_iterator i =
             mes.begin(), eoi = mes.end(); i != eoi; ++i) {
-        _logger->info("Metalink: Queueing %s for download as a member.",
+        logger_->info("Metalink: Queueing %s for download as a member.",
                       (*i)->getPath().c_str());
-        _logger->debug("originalName = %s", (*i)->metaurls[0]->name.c_str());
+        logger_->debug("originalName = %s", (*i)->metaurls[0]->name.c_str());
         (*i)->reorderResourcesByPriority();
         std::vector<std::string> uris;
         std::for_each((*i)->resources.begin(), (*i)->resources.end(),
